/*******************************************************************************
 * Define
 ******************************************************************************/
char SSID[32];
char PASSWORD[32];

char tempStr[128];

#define IP_THINGSPEAK "184.106.153.149"
#define TRUE 1
#define FALSE 0
#define WIFI_LCD_DEBUG 1

/*******************************************************************************
 * Zone de définition des variables globales
 ******************************************************************************/
#define NB_MOTS_CLES 8
char ListeMotsCles[NB_MOTS_CLES][12] = { "OK", "invdalid", "link error","+IPD", ">","Linked","192.168.0.","/"};
uint8_t iMax[NB_MOTS_CLES] = {2, 5, 10, 4, 1, 6, 10, 1};
#define CLE_OK 1
#define CLE_ready 2
#define CLE_link_error 4
#define CLE_IPD 8
#define CLE_sup 16
#define CLE_Linked 32
#define CLE_IP 64
#define CLE_SLASH 128
uint8_t retour;

/*******************************************************************************
  Fonction : SerialLoop
  Entrées :
    delai : en ms
    Find : id du mot clé à détecter
*******************************************************************************/
uint8_t SerialLoop(uint32_t delai, uint8_t Find)
{
    uint32_t heureDebut = millis();
    uint8_t i[NB_MOTS_CLES];
    uint8_t inByte, j;

	for (j=0;j<NB_MOTS_CLES;j++) i[j] = 0;

    while(((millis()-heureDebut) < delai))
    {
        // read from port 1, send to port 0:
        if (Serial3.available())
        {
            inByte = Serial3.read();
			for (j=0; j<NB_MOTS_CLES; j++)
			{
				if ((ListeMotsCles[j][i[j]] == inByte) && ((Find & (1<<j)) != 0))
				{
					i[j]++;
					if (i[j] == iMax[j])
					{
						#ifdef WIFI_LCD_DEBUG
						Serial.write(inByte);
						//Serial.println("\n");
						Serial.println("\n=TROUVEE!");
						#endif
						return (TRUE);
					}
				}
				else i[j] = 0;
			}
	        #ifdef WIFI_LCD_DEBUG
            Serial.write(inByte);
		    #endif
        }
    }
    return(FALSE);
}

/*******************************************************************************
  Fonction : SerialLoopStr
  Entrées :
    delai : en ms
    endMes : mot à repérer pour terminer la communication
    Find : indique s'il faut détecter le mot endMes
*******************************************************************************/
uint8_t SerialLoopStr(uint32_t delai, char * endMes, uint8_t Find)
{
    uint32_t heureDebut = millis();
    uint8_t iMax = strlen(endMes);
    uint8_t i=0;
    uint8_t inByte;

    while((millis()-heureDebut) < delai)
    {
        // read from port 1, send to port 0:
        if (Serial3.available())
        {
            inByte = Serial3.read();
            if ((endMes[i] == inByte) && Find)
            {
                i++;
                if (i == iMax)
                {
					#ifdef WIFI_LCD_DEBUG
                    Serial.write(inByte);
                    //Serial.println("\n");
                    Serial.println("\n=TROUVEE!");
					#endif
                    return (TRUE);
                }
            }
            else i = 0;
	        #ifdef WIFI_LCD_DEBUG
            Serial.write(inByte);
		    #endif
        }
    }
    return(FALSE);
}


/*******************************************************************************
  Fonction : WifiInit()
  Entrées :
  Sorties :
    retour false si erreur d'initialisation du module ESP8266
*******************************************************************************/
uint8_t WifiInit()
{
    Serial3.print("AT+RST\r\n");

    retour = SerialLoop(5000, CLE_ready);
    if (retour == 0) return 0;

    Serial3.print("AT+CWMODE=3\r\n");
    SerialLoop(5000, CLE_OK);
    return(1);
}

/*******************************************************************************
  Fonction : WifiAPConnect
  Entrées :
    ssid : chaîne de caractères SSID du réseau sur lequel se connecter
    passWord : mot de passe
  Sorties :
    TRUE si tout s'est bien passé
*******************************************************************************/
uint8_t WifiAPConnect(char * ssid, char * passWord)
{
    tempStr[0] = 0;
	sprintf(tempStr, "AT+CWJAP=\"%s\",\"%s\"\r\n", ssid, passWord);
    Serial3.print(tempStr);
    retour = SerialLoop(20000, CLE_OK);
	if (retour == 0) return FALSE;

    Serial3.print("AT+CIPMODE=0\r\n");
    if (SerialLoop(5000, CLE_OK))
    {
		Serial3.print("AT+CIFSR\r\n");
		SerialLoop(5000, CLE_OK);
    }
    else return FALSE;
    return(TRUE);
}

/*******************************************************************************
  Fonction : WifiTCPConnect
  Entrées :
    ip à contacter, port à utiliser
  Sorties :
    TRUE si tout est ok
*******************************************************************************/
uint8_t WifiTCPConnect(char * ip, uint16_t port)
{
    tempStr[0] = 0;
	sprintf(tempStr, "AT+CIPSTART=\"TCP\",\"%s\",%ld\r\n", ip, port);
    Serial3.print(tempStr);
    if (SerialLoop(2000, CLE_OK))
    {
	    return(TRUE);
    }
    else 
    {
        return FALSE;
    }

    return(TRUE);
}

/*******************************************************************************
  Fonction : WifiTCPServer
  Entrées :
    port à utiliser
  Sorties :
    TRUE si tout est ok
*******************************************************************************/
uint8_t WifiTCPServer(uint32_t port)
{
    tempStr[0] = 0;
    strcat(tempStr, "AT+CIPMUX=1\r\n");

	//UART_RX_Vide();
    // On vide le buffer de caractères de Serial3
    while(Serial3.available()) Serial3.read();

    Serial3.print(tempStr);
    if (SerialLoop(10000, CLE_OK))
    {
    }
    else
    {
        return FALSE;
    }

    sprintf(tempStr, "AT+CIPSERVER=1,%ld\r\n", port);

    Serial3.print(tempStr);
    SerialLoop(5000, CLE_OK);

    return(TRUE);
}

/*******************************************************************************
  Fonction : WifiTCPSend
  Entrées :
    mes : message à envoyer
  Sorties :
    TRUE si tout s'est bien passé
*******************************************************************************/
uint8_t WifiTCPSend(char * mes)
{
	uint8_t len = strlen(mes)+1;
    Serial3.print("AT+CIPSEND=");

    Serial3.print(strlen(mes)+1);
    Serial3.print("\r\n");

    if (SerialLoop(10000, CLE_sup))
    {
    }
    else
    {
        return FALSE;
    }

    Serial3.print(mes);
    Serial3.print("\r\n");
    if (SerialLoop(3000, CLE_OK))
    {
		return(TRUE);
    }
    else
    {
        return FALSE;
    }
}

/*******************************************************************************
  Fonction : WifiTCPAnswer
  Entrées :
    mes : message à envoyer
    Id : Id du canal de connexion
  Sorties :
    TRUE si tout s'est bien passé
*******************************************************************************/
uint8_t WifiTCPAnswer(char * mes, uint8_t Id)
{
	sprintf(tempStr, "AT+CIPSEND=%c,%d\r\n", Id, strlen(mes)+1);
    Serial3.print(tempStr);
    retour = SerialLoop(10000, CLE_sup);
	if (retour == 0) return FALSE;
    

    Serial3.print(mes);
    Serial3.print("\r\n");
    if (SerialLoop(10000, CLE_OK))
    {
    }
    else
    {
        return FALSE;
    }
}

/*******************************************************************************
  Fonction : WifiTCPSend
  Entrées :
    Id du canal à fermer
  Sorties :
    TRUE si tout s'est bien passé
*******************************************************************************/
uint8_t WifiTCPClose(uint8_t Id)
{
	if (Id!=0) sprintf(tempStr, "AT+CIPCLOSE=%c\r\n", Id); else sprintf(tempStr, "AT+CIPCLOSE\r\n");
    Serial3.print(tempStr);
    SerialLoop(10000, CLE_OK);
}

/*******************************************************************************
  Fonction : WifiIsConnected
  Entrées :
  Sorties :
    TRUE si connecté ; false sinon
*******************************************************************************/
/*uint8_t WifiIsConnected()
{
    Serial3.print("AT+CIFSR\r\n");

    if (SerialLoopStr(10000, CLE_IP, TRUE))
    {

		SerialLoop(10000, CLE_OK);
		return(TRUE);
    }
    else
    {
        SerialLoop(10000, CLE_OK);
		return FALSE;
    }
}
*/

void WifiServer()
{
		WifiInit();
		WifiTCPServer(80);

}

void setup() {
	// put your setup code here, to run once:
	Serial.begin(9600);
	Serial3.begin(115200);

	// Initialisation du module en point d'accès
	WifiServer();
}

void loop() {
	uint8_t uiIn = 0;
	// put your main code here, to run repeatedly:
	if (Serial.available())
	{
		uiIn = Serial.read();
		Serial3.write(uiIn);
	}
	if (Serial3.available())
	{
		uiIn = Serial3.read();
		Serial.write(uiIn);
	}
}
